#ifndef BBC_CLOPTIONS_H
#define BBC_CLOPTIONS_H

using namespace std;

// constants for the different columns in "PARAM_DEFINED_ARGS"
const int PARAM_SHORT = 0;
const int PARAM_LONG = 1;
const int PARAM_DEFAULT = 2;
const int PARAM_TYPE = 3;
const int PARAM_LIST_TYPE = 4;
const int PARAM_SUBTAG = 5;
const int PARAM_VISIBLE = 6;
const int PARAM_CHANGEABLE = 7;

// constants for the different types
const string PARAM_NUMBER = "number";
const string PARAM_STRING = "string";
const string PARAM_BOOL = "boolean";

// constants for the boolean values
const string PARAM_TRUE = "true";
const string PARAM_FALSE = "false";

// constants for normal and list type
const string PARAM_LIST = "list";
const string PARAM_NORMAL = "normal";

// constants for legalnumbers and null-strings
const string PARAM_NULLS = "";
const string PARAM_LEGALNUMBERCHARS = "0123456789-.eEdD";

// Command line parameters, in order 1) short name, 2) long name,
// 3) default value 4) Value type, 5) if command is of list type
// 5) if parameter is of list type, what is the subtags name

// All strings, end with row of all null strings (PARAM_NULLS)
// Allowed types: PARAM_NUMBER for all kinds of numerical values,
// PARAM_STRING for strings and PARAM_BOOL for booleans
// Only one of each parameter!



// This is the string that is supposed to be in the root element in order to
// match a <APP_NAME> with the parameters!
const string APP_NAME = "ranker";

// This is the name of the sub-section holding the Parameter information
const string PARAM_TAG = "parameters";

// The allowed arguments
static string PARAM_DEFINED_ARGS[][8] = 
{
  // General params
{ "-f"       ,  "-ff"                       , ""                , PARAM_STRING  , PARAM_NORMAL, "", PARAM_TRUE, PARAM_TRUE },
{ "-o"       ,  "-of"                       , "out.xml"         , PARAM_STRING  , PARAM_NORMAL, "", PARAM_TRUE, PARAM_TRUE },
{ "-hf"      ,  "-holy_file"                , ""                , PARAM_STRING  , PARAM_NORMAL, "", PARAM_TRUE, PARAM_TRUE },
{ "-pf"      ,  "-pattern_file"             , ""                , PARAM_STRING  , PARAM_NORMAL, "", PARAM_TRUE, PARAM_TRUE },
{ "-lf"      ,  "-log_file"                 , "log.txt"         , PARAM_STRING  , PARAM_NORMAL, "", PARAM_TRUE, PARAM_TRUE },
{ "-s"       ,  "-seed"                     , "0"               , PARAM_NUMBER  , PARAM_NORMAL, "", PARAM_TRUE, PARAM_TRUE },
{ "-mC"      ,  "-minCount"                 , "10"              , PARAM_NUMBER  , PARAM_NORMAL, "", PARAM_TRUE, PARAM_TRUE },
{ "-mL"      ,  "-minLength"                , "0.02"            , PARAM_NUMBER  , PARAM_NORMAL, "", PARAM_TRUE, PARAM_TRUE },
{ "-mS"      ,  "-minSupport"               , "-1"              , PARAM_NUMBER  , PARAM_NORMAL, "", PARAM_TRUE, PARAM_TRUE },
{ "-h"       ,  "-help"                     , PARAM_FALSE       , PARAM_BOOL    , PARAM_NORMAL, "", PARAM_TRUE, PARAM_TRUE },
{ "-v"       ,  "-version"                  , PARAM_FALSE       , PARAM_BOOL    , PARAM_NORMAL, "", PARAM_TRUE, PARAM_TRUE },
{ "-rmvG"    ,  "-removeGroupNames"                , PARAM_FALSE       , PARAM_BOOL    , PARAM_NORMAL, "", PARAM_TRUE, PARAM_TRUE },
{ "-addG"    ,  "-addGroupNames"                 , PARAM_FALSE       , PARAM_BOOL    , PARAM_NORMAL, "", PARAM_TRUE, PARAM_TRUE },
{ "-showG"    ,  "-showGroups"               , PARAM_FALSE       , PARAM_BOOL    , PARAM_NORMAL, "", PARAM_TRUE, PARAM_TRUE },
{ "-shLS"    ,  "-showLengthAndSupport"           , PARAM_FALSE       , PARAM_BOOL    , PARAM_NORMAL, "", PARAM_TRUE, PARAM_TRUE },
{ PARAM_NULLS,  PARAM_NULLS                 , PARAM_NULLS       , PARAM_NULLS   , PARAM_NULLS , PARAM_NULLS, PARAM_NULLS, PARAM_NULLS }
};

#endif //BBC_CLOPTIONS_H

